package de.hsbo.fbv.bmg.vectors;

import Jama.Matrix;

/**
 * Basisklasse fr Geraden und Strecken, behandelt bevorzugt 2-dim. und 3-dim. Vektoren
 * entsprechend den Algorithmen der analytischen Geometrie.
 * Intern angewendet wird die vektorielle Gleichung:
 * v(u) = s*(1-u) + e*u 
 * @author Zimmermann, HS Bochum, 2009
 *
 */
public class Line {
	
	/**
	 * Attribute Ortsvektoren fr Startpunkt und Endpunkt
	 */
	protected Vector s;		// Startpunkt-Vektor
	protected Vector e;		// Endpunkt-Vektor
	protected int dim = 0;	// Dimensionen der Punkt-Vektoren
	
	/**
	 * Konstruktor, bergibt Start- und Endpunkt
	 * @param sp - Startpunkt-Vektor
	 * @param ep - Endpunkt-Vektor
	 */
	public Line(Vector sp, Vector ep) {
		try {
			if(sp.dim() == ep.dim()) {
				this.s = sp;
				this.e = ep;
				this.dim = sp.dim();
			} else {
				throw new VectorDimensionException("dimensions differ");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			this.s = null;
			this.e = null;
		}
	}
	
	/**
	 * Gibt den Vektor des Startpunktes zurck.
	 * @return Startpunkt-Vektor
	 */
	public Vector getStart() {
		return s;
	}
	
	/**
	 * Gibt den Vektor des Endpunktes zurck.
	 * @return Endpunkt-Vektor
	 */
	public Vector getEnd() {
		return e;
	}
	
	/**
	 * Berechnet den Geraden-(Strecken-)Punkt mit dem Parameter u.
	 * Der Punkt befindet sich im Streckenabschnitt, wenn u zwischen 0 und 1 liegt.
	 * Sind die Dimensionen der Punktvektoren inkonstent, wird eine Exception geworfen.
	 * @param u - Parameter
	 * @return Punktvektor oder null
	 */
	public Vector getPoint(double u) {
		try {
			if (this.dim != 0) {
				Vector res = (s.scale((1 - u)).add((e.scale(u))));
				return res;
			} else
				throw new VectorDimensionException("dimensions differ");
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Berechnet den Geraden-(Strecken-)Parameter u des Lotfupunktes von Vektor p.
	 * Die Projektion von p befindet sich im Streckenabschnitt, wenn u zwischen 0 und 1 liegt.
	 * Sind die Dimensionen des Punktes und der Linie inkonstent, wird eine Exception geworfen.
	 * @param p - Vektor
	 * @return Parameter des Lotfupunkt-Vektors oder NaN
	 */
	public double getParameter(Vector p) {
		try {
			if(this.dim == p.dim()) {
				Vector sp = p.sub(s);
				Vector se = e.sub(s);
				return sp.mul(se)/(se.length()*se.length());
			}
			else
				throw new VectorDimensionException("dimensions differ");
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return Double.NaN;
		}	
	}
	
	/**
	 * Berechnet den Abstand zwischen der Geraden(Strecke) und dem Punkt p.
	 * Sind die Dimensionen des Punktes und der Linie inkonstent, wird eine Exception geworfen.
	 * @param p - Vektor
	 * @return Abstand oder NaN
	 */	
	public double getDistance(Vector p) {
		try {
			if (this.dim == p.dim()) {
				Vector fp = this.getFootPoint(p);
				Vector diff = p.sub(fp);
				return diff.length();
			} else
				throw new VectorDimensionException("dimensions differ");
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return Double.NaN;
		}
	}
	
	/**
	 * Berechnet den Lotfupunkt (Projektion) des Punktes p auf die Gerade(Strecke).
	 * Sind die Dimensionen des Punktes und der Linie inkonstent, wird eine Exception geworfen.
	 * @param p - Vektor
	 * @return Lotfupunkt-Vektor oder null
	 */
	public Vector getFootPoint(Vector p) {
		try {
			if(this.dim == p.dim()) {
				return (this.getPoint(this.getParameter(p)));
			}
			else
				throw new VectorDimensionException("dimensions differ");
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/** 
	 * Berechnet die Parameter des Schnittpunktes von dieser Geraden und der Geraden l.
	 * Zur Berechnung werden nur die beiden ersten Komponenten der Vektoren verwendet.
	 * Sind die Dimensionen der Linienpunkte inkonstent, wird eine Exception geworfen.
	 * @param l
	 * @return {Parameter von Line this, Parameter von Line l}
	 */
	public double[] crossParameter(Line l) {
		try {
			if(this.dim == l.dim) {
				Vector tr  = this.e.sub(this.s);
				Vector lr  = l.e.sub(l.s);
				Vector tl  = this.s.sub(l.s);
				double[][] x = new double[][] {{-tr.c[0], lr.c[0]}, {-tr.c[1], lr.c[1]}};
				double[][] b = new double[][] {{tl.c[0]},{tl.c[1]}};
				Matrix xm = new Matrix(x);
				Matrix xb = new Matrix(b);
				Matrix xl = xm.solve(xb);
				return xl.getColumnPackedCopy();
			}
			else throw new VectorDimensionException("dimensions differ");
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Berechnet den normierten Richtungsvektor zwischen Start- und Endpunkt.
	 * @return normierter Richtungsvektor
	 */
	public Vector getDirection() {
		Vector r = this.e.sub(this.s);
		double length = r.length();
		r = r.scale(1/length);
		return r;
	}
	
	/**
	 * Berechnet die Lnge des Richtungsvektors zwischen Start- und Endpunkt.
	 * @return Lnge des Richtungsvektors
	 */
	public double getLength() {
		Vector r = this.e.sub(this.s);
		double length = r.length();
		return length;
	}
		
}	// Ende der Klasse


