package de.hsbo.fbv.bmg.vectors;

/**
 * Basisklasse fr Vektoren, behandelt bevorzugt 2-dim. und 3-dim. Vektoren
 * entsprechend den Algorithmen der analytischen Geometrie.
 * @author Zimmermann, HS Bochum, 2009
 * 
 */
public class Vector {

	/**
	 * Attribut Koordinaten-Array c
	 */
	protected double[] c;
	
	/**
	 * Konstruktor, bergibt ein Koordinaten-Array.
	 * Lst eine Exception aus, wenn das Array leer ist (null-Pointer).
	 * @param coords
	 */
	public Vector(double[] coords) {
		try {
			if(coords==null)
				throw new VectorDimensionException("empty coordinates array");
			else if(coords.length<2)
				throw new VectorDimensionException("number of components less than two");
			else
				this.c = coords;
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			this.c = null;
		}
	}
	
	/**
	 * Gibt das Koordinaten-Array zurck.
	 * @return Koordinaten-Array
	 */
	public double[] getCoords() {
		return this.c;
	}
	
	/**
	 * Setzt im Koordinaten-Array das Element mit dem Index i auf den Wert v.
	 * @return 
	 */
	public void setCoord(int i, double v) {
		this.c[i] = v;
	}
	
	/**
	 * Gibt die Dimension des Vektors zurck (z.B. 2-dim. oder 3-dim.)
	 * @return Dimension
	 */
	public int dim() {
		return c.length;
	}
	
	/**
	 * Addiert diesen Vektor und Vektor p.
	 * Lst eine Exceptions aus, wenn die Dimensionen ungleich sind.
	 * @param p 
	 * @return Vektorsumme oder null
	 */
	public Vector add(Vector p) {
		try {
			if(this.c.length==p.c.length) {
				double[] res = new double[this.c.length];
				for(int i=0; i<this.c.length; i++)
					res[i] = this.c[i] + p.c[i];
				return new Vector(res);
			}
			else {
				throw new VectorDimensionException("dimensions differ");		
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Subtrahiert von diesem Vektor den Vektor p.
	 * Lst eine Exceptions aus, wenn die Dimensionen ungleich sind.
	 * @param p
	 * @return Vektordifferenz oder null
	 */
	public Vector sub(Vector p) {
		try {
			if(this.c.length==p.c.length) {
				double[] res = new double[this.c.length];
				for(int i=0; i<this.c.length; i++)
					res[i] = this.c[i] - p.c[i];
				return new Vector(res);
			}
			else {
				throw new VectorDimensionException("dimensions differ");		
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	 * Multipliziert diesen Vektor und Vektor p skalar.
	 * Lst eine Exceptions aus, wenn die Dimensionen ungleich sind.
	 * @param p
	 * @return skalares Produkt der Vektoren oder NaN
	 */
	public double mul(Vector p) {
		try {
			if (this.c.length == p.c.length) {
				double res = 0;
				for (int i = 0; i < this.c.length; i++)
					res = res + this.c[i] * p.c[i];
				return res;
			} else {
				throw new VectorDimensionException("dimensions differ");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return Double.NaN;
		}
	}
	
	/**
	 * Multipliziert diesen Vektor mit dem Faktor f.
	 * @param f
	 * @return skalierter Vektor
	 */
	public Vector scale(double f) {
		double[] res = new double[this.c.length];
		for(int i=0; i<this.c.length; i++)
			res[i] = this.c[i] * f;
		return new Vector(res);
	}
	
	/**
	 * Berechnet den Lotrichtungsvektor zu diesem (2-dim.) Vektor.
	 * Lst eine Exceptions aus, wenn der Vektor nicht 2-dim. ist.
	 * @return Lotrichtungsvektor (2-dim.) oder null
	 */
	public Vector ortho() {
		try {
			if(this.c.length == 2) {
				double[] res = new double[] {-c[1], c[0]};
				return new Vector(res);
			} else {
				throw new VectorDimensionException("dimensions <> 2");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}	
	}
	
	/**
	 * Multipliziert diesen Vektor und Vektor p (beide 2-dim.) vektoriell.
	 * Das Ergebnis ist ein gerichteter Flcheninhalt der aufgespannten Vektoren.
	 * Lst eine Exceptions aus, wenn einer der Vektoren nicht 2-dim. sind.
	 * @param p
	 * @return gerichteter Flcheninhalt oder NaN
	 */
	public double vprod(Vector p) {
		try {
			if((this.c.length == 2) && (p.c.length == 2)) {
				double res = c[0]*p.c[1] - c[1]*p.c[0];
				return res;
			} else {
				throw new VectorDimensionException("dimensions <> 2");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return Double.NaN;
		}	
	}
	
	/**
	 * Berechnet den Lotrichtungsvektor auf diesem Vektor und Vektor p (beide 3-dim.).
	 * Das Ergebnis steht auf beiden Vektoren senkrecht, seine Lnge entspricht dem 
	 * Flcheninhalt der beiden aufgespannten Vektoren.
	 * Lst eine Exception aus, wenn einer der Vektoren nicht 3-dim. sind.
	 * @param p
	 * @return Vektorprodukt bzw. Flcheninhalt oder NaN
	 */
	public Vector ortho(Vector p) {
		try {
			if((this.c.length == 3) && (p.c.length == 3)) {
				double[] res = new double[] {
						 c[2]*p.c[3] - c[3]*p.c[2],
						-c[1]*p.c[3] - c[3]*p.c[1],						
						 c[0]*p.c[1] - c[1]*p.c[0]};
				return new Vector(res);
			} else {
				throw new VectorDimensionException("dimensions <> 3");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return null;
		}	
	}
	
	/**
	 * Berechnet das Spatprodukt aus diesem Vektor und den Vektoren p und q (alle 3-dim.).
	 * Das Ergebnis ist der gerichtete Rauminhalt der aufgespannten Vektoren. 
	 * Lst eine Exception aus, wenn einer der Vektoren nicht 3-dim. sind.
	 * @param p
	 * @param q
	 * @return gerichteter Rauminhalt oder NaN
	 */
	public double vprod(Vector p, Vector q) {
		try {
			if((this.c.length == 3) && (p.c.length == 3) && (q.c.length == 3)) {
				Vector r = this.ortho(p);
				return r.mul(q);
			} else {
				throw new VectorDimensionException("dimensions <> 3");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			return Double.NaN;
		}	
	}

	/**
	 * Berechnet die Lnge dieses Vektors.
	 * @return Lnge
	 */
	public double length() {
		double res = 0;
		for(int i=0; i<this.c.length; i++)
			res = res + this.c[i] * this.c[i];
		return Math.sqrt(res);
	}
	
	/**
	 * Textausgabe
	 */
	public String toString() {
		String s = "(";
		for(int i=0; i<this.c.length-1; i++)
			s = s + this.c[i] + " ";
		s = s + this.c[this.c.length-1] + ")";
		return s;
	}
	
}	// Ende der Klasse
